package edu.upvehu.gbg;

import java.io.Closeable;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Set;

public class DBMummec2223 implements Closeable{
   private static DBMummec2223 theInstance=null;
   private final Connection con;
   private static final String DATABASE_NAME = "MUMMEC2223";
   
   private DBMummec2223() throws SQLException{
      con=DriverManager.getConnection("jdbc:mysql://localhost:3306/?serverTimezone=UTC","german","chicle");
 }
   
       void instanciaEsquema() throws SQLException{
            //Se genera la base de datos destruyéndola previamente si existe. 
            con.createStatement().executeUpdate("DROP DATABASE IF EXISTS "+DATABASE_NAME+";");
            con.createStatement().executeUpdate("CREATE DATABASE "+DATABASE_NAME+";");
            con.createStatement().executeUpdate("USE "+DATABASE_NAME+";");
            
            con.setAutoCommit(false);
            con.createStatement().executeUpdate("CREATE TABLE papersreferences ( ref_paper INTEGER, cited_ref_paper INTEGER, PRIMARY KEY(ref_paper,cited_ref_paper) );");
            con.createStatement().executeUpdate("CREATE TABLE publication ( ref_publication INTEGER PRIMARY KEY, title VARCHAR(200) );");
            con.createStatement().executeUpdate("CREATE TABLE papersauthors ( ref_paper INTEGER, ref_author INTEGER, PRIMARY KEY(ref_paper,ref_author) );");
            con.createStatement().executeUpdate("CREATE TABLE authorafiliation ( ref_author INTEGER, ref_afiliation INTEGER, PRIMARY KEY(ref_author, ref_afiliation) );");
            con.createStatement().executeUpdate("CREATE TABLE afiliations ( ref_afiliation INTEGER PRIMARY KEY, university VARCHAR(65), webpage VARCHAR(30), country VARCHAR(15) );");
            con.createStatement().executeUpdate("CREATE TABLE papersinfo ( ref_paper INTEGER PRIMARY KEY, title VARCHAR(200), publiyear INTEGER, ref_publication INTEGER, url VARCHAR(200));");
            con.createStatement().executeUpdate("CREATE TABLE authors ( ref_author INTEGER PRIMARY KEY, name VARCHAR(25) );" );
            con.commit();
    }
   
   public static DBMummec2223 getInstance() throws SQLException{
       if (theInstance==null) theInstance=new DBMummec2223();
       return theInstance;
   }
   
    //PARA autores, publicaciones
    void arrayOfStrings2DB(String[] valores,String tableName) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into "+tableName+" values (?,?);");
        int i=0; for (String s:valores) {
            ps.setInt(1, ++i);
            ps.setString(2, s);
            ps.addBatch();
        }
        flush(ps);
    }
   
    void filiaciones2DB(String[] valores) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into afiliations values (?,?,?,?);");
        int i=0; for (String s:valores) {
            String[] campos=s.split("\\s*,\\s*");
            ps.setInt(1, ++i);
            ps.setString(2, campos[0]);
            ps.setString(3, campos[1]);
            ps.setString(4, campos[2]);
            ps.addBatch();
        }
        flush(ps);
    }   
 
     void autorFiliacion2DB(Set<String> afset) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into authorafiliation values (?,?);");
        for (String s:afset) {
            String[] campos=s.split("<SEP>");
            ps.setInt(1, Integer.parseInt(campos[0]));
            ps.setInt(2, Integer.parseInt(campos[1]));
            ps.addBatch();
            }
        flush(ps);
    }
     
    void paperAuth2DB(Set<String> paperAuth) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into papersauthors values (?,?);");
        for (String s:paperAuth) {
            String[] campos=s.split("<SEP>");
            ps.setInt(1, Integer.parseInt(campos[0]));  //Numero de referencia
            ps.setInt(2, Integer.parseInt(campos[1]));   //Autor (indice)
            ps.addBatch();
            }
        flush(ps);
    }

    void info2DB(Set<String> info) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into papersinfo values (?,?,?,?,?);");
        for (String s:info) {
            String[] campos=s.split("<SEP>");
            ps.setInt(1, Integer.parseInt(campos[0]));  //Numero de referencia
            ps.setString(2, campos[1]);                 //Titulo
            ps.setInt(3, Integer.parseInt(campos[2]));  //Año
            ps.setInt(4, Integer.parseInt(campos[3]));  //Publicacion (indice)
            ps.setString(5, campos[4]);                 //Link
            ps.addBatch();
            }
        flush(ps);
    }

    void refs2DB(Set<String> refs) throws SQLException {
        PreparedStatement ps=con.prepareStatement("insert into papersreferences values (?,?);");
        for (String s:refs) {
            String[] campos=s.split("<SEP>");
            for (String ref:campos[1].split("\\s+")) {      //cada linea de referencias tiene varias a separar
                ps.setInt(1, Integer.parseInt(campos[0]));
                ps.setInt(2, Integer.parseInt(ref));
                ps.addBatch();
            }
        }
        flush(ps);
    }
    
    //FLUSH
    private void flush(PreparedStatement ps) throws SQLException {ps.executeBatch();con.commit();}

    // Implementación del interface Closeable
    @Override
    public void close() throws IOException {
       try {
           if (con!=null) con.close();
       } catch (SQLException ex) {
           throw new IOException("No puedo cerrar la conexion con la base de datos",ex);
       }
    }

}
