package edu.upvehu.gbg;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import javax.swing.JOptionPane;

public class Solucion {
    private static final String URL="https://gtts.ehu.es/German/Docencia/assets/DB.txt";

    private Solucion(){}
    
    public static void main(String[] args) {
        //Hacemos todo dentro de un try con dos recursos: el fichero remoto y la base de datos
        try ( BufferedReader br = new BufferedReader(new InputStreamReader(new URL(URL).openStream()));
            DBMummec2223 db = DBMummec2223.getInstance();) {
            //Leemos el fichero remoto (todo se hace en el chequeo de la expresión, por lo que no hay cuerpo del while 
            while (Referencia.addLine(br.readLine()) != null);

            //(Re)generamos la base de datos
            db.instanciaEsquema();

            //Generamos las tablas que funcionarán como índices: publicaciones, autores y filiaciones
            Set<String> pub = new TreeSet<>(), auth = new TreeSet<>(), fil = new TreeSet<>() ;
            for (Referencia r : Referencia.getALMACEN().values()) {      //Recorriendo todos los registros (los valores del mapa)
                pub.add(r.getPublicacion());                             //Se añaden las publicaciones a su conjunto
                auth.addAll(r.getAutores());                             //Se añaden los autores a su conjunto
                fil.addAll(r.getFiliaciones());                          //Se añaden las filiaciones a su conjunto
            }
            String[] publicaciones = pub.toArray(new String[pub.size()]);//(por conveniencia posterior se pasa el conjunto a un array)
            String[] autores = auth.toArray(new String[auth.size()]);    //(por conveniencia posterior se pasa el conjunto a un array)
            String[] filiaciones = fil.toArray(new String[fil.size()]);  //(por conveniencia posterior se pasa el conjunto a un array)
            db.arrayOfStrings2DB(publicaciones, "publication");
            db.arrayOfStrings2DB(autores, "authors");
            db.filiaciones2DB(filiaciones);

            //Generamos las tablas de autor-filiación papel-autor, info y de referencias    
            Set<String> afset = new HashSet<>(), paperAuth = new HashSet<>(), info = new HashSet<>(), refs = new HashSet<>();
            for (Map.Entry<Integer, Referencia> e : Referencia.getALMACEN().entrySet()){
                Integer k=e.getKey();
                Referencia v=e.getValue();
                for (String af:v.getAutorFiliacionList()){
                    String[] afsplit=af.replaceFirst("(.*?)\"(.*?)\".*", "$1<SEP>$2").split("<SEP>");
                    int authIndex=1+Arrays.binarySearch(autores, afsplit[0].trim());
                    afset.add(authIndex +"<SEP>"+ (1+Arrays.binarySearch(filiaciones, afsplit[1].trim())));
                    paperAuth.add(k +"<SEP>"+ authIndex);
                }
                info.add(k+"<SEP>"+v.getTitulo() +"<SEP>"+ v.getAnio() +"<SEP>"+ (1+Arrays.binarySearch(publicaciones,v.getPublicacion())) +"<SEP>"+ v.getLink());
                if (v.getReferencias()!=null) refs.add(k +"<SEP>"+ v.getReferencias());
            }    
            db.autorFiliacion2DB(afset);
            db.paperAuth2DB(paperAuth);            
            db.info2DB(info);
            db.refs2DB(refs);            


        } catch (IOException | InvalidFieldException | SQLException ex){
            JOptionPane.showMessageDialog(null, "<html>Se ha producido un error:<br>"+ex);
        }
    }

}
